#include "Status.h"
#include "Vram.h"
#include "Print.h"
#include "Chars.h"
#include "Sprite.h"

constexpr byte Char_Remain = Char_Fighter;

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus()
{
    PrintS(Vram + VramRowSize * 0 + WindowWidth * VramStep, "SCORE");
    PrintS(Vram + VramRowSize * 3 + WindowWidth * VramStep, "HIGH");
    PrintS(Vram + VramRowSize * 4 + WindowWidth * VramStep, "SCORE");
    PrintS(Vram + VramRowSize * 7 + WindowWidth * VramStep, "STAGE");
    PrintByteNumber2(Vram + VramRowSize * 8 + VramWidth - 2 * VramStep, CurrentStage + 1);
    PrintRemain();
    PrintScore();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramRowSize * 1 + WindowWidth * VramStep, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + VramRowSize * 5 + WindowWidth * VramStep, HiScore);
    PrintC(pVram, '0');
}

void PrintRemain()
{
    ptr<byte> pVram;
    pVram = Vram + VramRowSize * 21 + (WindowWidth + 1) * VramStep;
    if (RemainCount > 1) {
        byte i;
        i = RemainCount - 1;
        if (i > 2) {
            pVram = Put2C(pVram, Char_Remain, 0x47);
            pVram = Erase2(pVram);
            pVram += VramRowSize - 2;
            pVram = Put(pVram, i + 0x10, 0x47);
            pVram += 1 - VramRowSize;
        }
        else {
            do {
                pVram = Put2C(pVram, Char_Remain, 0x47);
                --i;
            } while (i > 0);
        }
    }
    if (RemainCount < 3) {
        pVram = Erase2(pVram);
    }
}


void PrintGameOver()
{
    constexpr byte Left = (WindowWidth - 8) / 2;
    PrintS(Vram + VramRowSize * (VramHeight / 2) + Left * VramStep, "GAME OVER");
}

void Title()
{
    ClearScreen(); 
    HideAllSprites();
    // UpdateSprites();
    PrintStatus();
    {
        static const byte[] TitleBytes = {
            //	G
            0x00, 0xf6, 0xee, 0xe3, 
            0xe8, 0xf2, 0x94, 0x94, 
            0xfa, 0xe3, 0x00, 0x8e, 
            0x00, 0xfa, 0xee, 0xee, 
            //	U
            0xe8, 0xf2, 0x00, 0x8e, 
            0xe8, 0xf2, 0x00, 0x8e, 
            0xe8, 0xf2, 0x00, 0x8e, 
            0x00, 0xee, 0xee, 0xf8, 
            //	N
            0xe8, 0xe3, 0x00, 0x8e, 
            0xe8, 0x8e, 0xe3, 0x8e, 
            0xe8, 0xf2, 0xe6, 0x8e, 
            0xfa, 0xf8, 0x00, 0xee, 
            //	T
            0xfa, 0xe6, 0xe4, 0xf8, 
            0x00, 0xe8, 0xf2, 0x00, 
            0x00, 0xe8, 0xf2, 0x00, 
            0x00, 0xfa, 0xf8, 0x00, 
            //	U
            0xe8, 0xf2, 0x00, 0x8e, 
            0xe8, 0xf2, 0x00, 0x8e, 
            0xe8, 0xf2, 0x00, 0x8e, 
            0x00, 0xee, 0xee, 0xf8, 
            //	S
            0xe1, 0xe4, 0xee, 0xe3, 
            0xfa, 0xe3, 0x94, 0xf3, 
            0xe1, 0xf3, 0x00, 0x8e, 
            0x00, 0xee, 0xee, 0xf8, 
        };
        constexpr byte LogoLength = 6;
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 7 + (WindowWidth - LogoLength * 4)  * VramStep / 2;
        p = TitleBytes;
        repeat (LogoLength) {
            repeat (4) {
                repeat (4) {
                    pVram = Put(pVram, *p, 0x05);
                    ++p;
                }
                pVram += VramRowSize - 4 * VramStep;
            }
            pVram += 4 * VramStep - VramRowSize * 4;
        }
    }
    PrintS(Vram + VramRowSize * 19 + (WindowWidth - 14) * VramStep / 2, "PUSH SPACE KEY");
    PrintS(Vram + VramRowSize * 20 + (WindowWidth - 14) * VramStep / 2, "OR RETURN KEY");
    PrintS(Vram + VramRowSize * 22 + (WindowWidth - 12) * VramStep, "INUFUTO 2022");
    // {
    //     byte c;
    //     ptr<byte> pVram;
    //     pVram = Vram;
    //     c = 0;
    //     repeat (0) {
    //         pVram = Put(pVram, c);
    //         ++c;
    //         if ((c & 15) == 0) {
    //             pVram += VramRowSize - 16;
    //         }
    //     }
    // //     // UpdateEnemyChars();
    // //     // SwitchChars();
    // //     // while (true) {
    // //     //     ++EnemyRowLeft;
    // //     //     UpdateEnemyChars();
    // //     //     SwitchChars();
    // //     //     WaitTimer(30);
    // //     // }
    // }
}


void AddScore(word pts)
{
    Score += pts;
    if (Score > HiScore) {
        HiScore = Score;
    }
    PrintScore();
}
